# -*- coding: utf-8 -*-

"""
test_i2c_lcd
----------------------------------

Tests for `i2c_lcd` module.
"""


import sys
import unittest

from i2c_lcd import i2c_lcd


class TestI2c_lcd(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_000_something(self):
        pass
